/********************************************************************************
** Form generated from reading UI file 'QvkCameraSingle_wl.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QVKCAMERASINGLE_WL_H
#define UI_QVKCAMERASINGLE_WL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_QvkCameraSingle_wl
{
public:
    QHBoxLayout *horizontalLayout_2;
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_6;
    QComboBox *comboBoxCameraFPS;
    QHBoxLayout *horizontalLayout;
    QCheckBox *checkBoxCameraOnOff;
    QHBoxLayout *horizontalLayout_4;
    QComboBox *comboBoxCameraResolution;
    QHBoxLayout *horizontalLayout_3;
    QComboBox *comboBoxCameraPixelformat;

    void setupUi(QWidget *QvkCameraSingle_wl)
    {
        if (QvkCameraSingle_wl->objectName().isEmpty())
            QvkCameraSingle_wl->setObjectName("QvkCameraSingle_wl");
        QvkCameraSingle_wl->resize(469, 48);
        QvkCameraSingle_wl->setWindowTitle(QString::fromUtf8("Form"));
        horizontalLayout_2 = new QHBoxLayout(QvkCameraSingle_wl);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, -1, 0);
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        comboBoxCameraFPS = new QComboBox(QvkCameraSingle_wl);
        comboBoxCameraFPS->setObjectName("comboBoxCameraFPS");

        horizontalLayout_6->addWidget(comboBoxCameraFPS);


        gridLayout->addLayout(horizontalLayout_6, 0, 4, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        checkBoxCameraOnOff = new QCheckBox(QvkCameraSingle_wl);
        checkBoxCameraOnOff->setObjectName("checkBoxCameraOnOff");
        checkBoxCameraOnOff->setText(QString::fromUtf8("CheckBox"));

        horizontalLayout->addWidget(checkBoxCameraOnOff);


        gridLayout->addLayout(horizontalLayout, 0, 0, 1, 1);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        comboBoxCameraResolution = new QComboBox(QvkCameraSingle_wl);
        comboBoxCameraResolution->setObjectName("comboBoxCameraResolution");
        comboBoxCameraResolution->setCurrentText(QString::fromUtf8(""));

        horizontalLayout_4->addWidget(comboBoxCameraResolution);


        gridLayout->addLayout(horizontalLayout_4, 0, 2, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        comboBoxCameraPixelformat = new QComboBox(QvkCameraSingle_wl);
        comboBoxCameraPixelformat->setObjectName("comboBoxCameraPixelformat");
        comboBoxCameraPixelformat->setCurrentText(QString::fromUtf8(""));
        comboBoxCameraPixelformat->setDuplicatesEnabled(true);

        horizontalLayout_3->addWidget(comboBoxCameraPixelformat);


        gridLayout->addLayout(horizontalLayout_3, 0, 1, 1, 1);

        gridLayout->setColumnStretch(0, 4);

        horizontalLayout_2->addLayout(gridLayout);


        retranslateUi(QvkCameraSingle_wl);

        QMetaObject::connectSlotsByName(QvkCameraSingle_wl);
    } // setupUi

    void retranslateUi(QWidget *QvkCameraSingle_wl)
    {
        (void)QvkCameraSingle_wl;
    } // retranslateUi

};

namespace Ui {
    class QvkCameraSingle_wl: public Ui_QvkCameraSingle_wl {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QVKCAMERASINGLE_WL_H
