/****************************************************************************
** Meta object code from reading C++ file 'QvkPlayerGst.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "src/playerGST/linux/QvkPlayerGst.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'QvkPlayerGst.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12QvkPlayerGstE_t {};
} // unnamed namespace

template <> constexpr inline auto QvkPlayerGst::qt_create_metaobjectdata<qt_meta_tag_ZN12QvkPlayerGstE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QvkPlayerGst",
        "signal_EOS",
        "",
        "value",
        "signal_duration",
        "duration_second",
        "signal_currentTime",
        "currentTime",
        "signal_mute",
        "bol",
        "signal_volume",
        "volume",
        "signal_pause",
        "slot_timer",
        "slot_discover_quit",
        "video",
        "audio",
        "slot_play",
        "slot_stop",
        "slot_pause",
        "slot_EOS"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signal_EOS'
        QtMocHelpers::SignalData<void(QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'signal_duration'
        QtMocHelpers::SignalData<void(qint64)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 5 },
        }}),
        // Signal 'signal_currentTime'
        QtMocHelpers::SignalData<void(qint64)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 7 },
        }}),
        // Signal 'signal_mute'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'signal_volume'
        QtMocHelpers::SignalData<void(qreal)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 11 },
        }}),
        // Signal 'signal_pause'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slot_timer'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_discover_quit'
        QtMocHelpers::SlotData<void(bool, bool)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 15 }, { QMetaType::Bool, 16 },
        }}),
        // Slot 'slot_play'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slot_stop'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slot_pause'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slot_EOS'
        QtMocHelpers::SlotData<void(QString)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QvkPlayerGst, qt_meta_tag_ZN12QvkPlayerGstE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QvkPlayerGst::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QvkPlayerGstE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QvkPlayerGstE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12QvkPlayerGstE_t>.metaTypes,
    nullptr
} };

void QvkPlayerGst::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QvkPlayerGst *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signal_EOS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->signal_duration((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 2: _t->signal_currentTime((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 3: _t->signal_mute((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->signal_volume((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 5: _t->signal_pause(); break;
        case 6: _t->slot_timer(); break;
        case 7: _t->slot_discover_quit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 8: _t->slot_play(); break;
        case 9: _t->slot_stop(); break;
        case 10: _t->slot_pause(); break;
        case 11: _t->slot_EOS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QvkPlayerGst::*)(QString )>(_a, &QvkPlayerGst::signal_EOS, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QvkPlayerGst::*)(qint64 )>(_a, &QvkPlayerGst::signal_duration, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QvkPlayerGst::*)(qint64 )>(_a, &QvkPlayerGst::signal_currentTime, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QvkPlayerGst::*)(bool )>(_a, &QvkPlayerGst::signal_mute, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QvkPlayerGst::*)(qreal )>(_a, &QvkPlayerGst::signal_volume, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QvkPlayerGst::*)()>(_a, &QvkPlayerGst::signal_pause, 5))
            return;
    }
}

const QMetaObject *QvkPlayerGst::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QvkPlayerGst::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QvkPlayerGstE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QvkPlayerGst::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void QvkPlayerGst::signal_EOS(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QvkPlayerGst::signal_duration(qint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QvkPlayerGst::signal_currentTime(qint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QvkPlayerGst::signal_mute(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QvkPlayerGst::signal_volume(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QvkPlayerGst::signal_pause()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
