/****************************************************************************
** Meta object code from reading C++ file 'mainWindow_wl.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "src/wayland/mainWindow_wl.h"
#include <QtGui/qtextcursor.h>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainWindow_wl.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QvkMainWindow_wlE_t {};
} // unnamed namespace

template <> constexpr inline auto QvkMainWindow_wl::qt_create_metaobjectdata<qt_meta_tag_ZN16QvkMainWindow_wlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QvkMainWindow_wl",
        "signal_newVideoFilename",
        "",
        "filename",
        "signal_beginRecordTime",
        "time",
        "slot_pre_portal_start",
        "slot_portal_start",
        "slot_portal_dialog_aborted",
        "slot_pre_start",
        "vk_fd",
        "vk_path",
        "slot_start_gst",
        "slot_stop",
        "slot_newVideoPath",
        "slot_videoFileSystemWatcherSetNewPath",
        "slot_log_folder",
        "slot_log_refresh",
        "slot_folder",
        "slot_Pause",
        "slot_Continue",
        "slot_snapshotHideBeforeRecording",
        "slot_pushButton_snapshot",
        "slot_handle_response_snapshot",
        "responseCode",
        "QVariantMap",
        "results",
        "slot_path_to_snapshot_folder"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signal_newVideoFilename'
        QtMocHelpers::SignalData<void(QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'signal_beginRecordTime'
        QtMocHelpers::SignalData<void(QString)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'slot_pre_portal_start'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_portal_start'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_portal_dialog_aborted'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_pre_start'
        QtMocHelpers::SlotData<void(QString, QString)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'slot_start_gst'
        QtMocHelpers::SlotData<void(QString, QString)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'slot_stop'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_newVideoPath'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_videoFileSystemWatcherSetNewPath'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_log_folder'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_log_refresh'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_folder'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_Pause'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_Continue'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_snapshotHideBeforeRecording'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_pushButton_snapshot'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slot_handle_response_snapshot'
        QtMocHelpers::SlotData<void(uint, QVariantMap)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::UInt, 24 }, { 0x80000000 | 25, 26 },
        }}),
        // Slot 'slot_path_to_snapshot_folder'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QvkMainWindow_wl, qt_meta_tag_ZN16QvkMainWindow_wlE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QvkMainWindow_wl::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QvkMainWindow_wlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QvkMainWindow_wlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QvkMainWindow_wlE_t>.metaTypes,
    nullptr
} };

void QvkMainWindow_wl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QvkMainWindow_wl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signal_newVideoFilename((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->signal_beginRecordTime((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->slot_pre_portal_start(); break;
        case 3: _t->slot_portal_start(); break;
        case 4: _t->slot_portal_dialog_aborted(); break;
        case 5: _t->slot_pre_start((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->slot_start_gst((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->slot_stop(); break;
        case 8: _t->slot_newVideoPath(); break;
        case 9: _t->slot_videoFileSystemWatcherSetNewPath(); break;
        case 10: _t->slot_log_folder(); break;
        case 11: _t->slot_log_refresh(); break;
        case 12: _t->slot_folder(); break;
        case 13: _t->slot_Pause(); break;
        case 14: _t->slot_Continue(); break;
        case 15: _t->slot_snapshotHideBeforeRecording(); break;
        case 16: _t->slot_pushButton_snapshot(); break;
        case 17: _t->slot_handle_response_snapshot((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 18: _t->slot_path_to_snapshot_folder(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QvkMainWindow_wl::*)(QString )>(_a, &QvkMainWindow_wl::signal_newVideoFilename, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QvkMainWindow_wl::*)(QString )>(_a, &QvkMainWindow_wl::signal_beginRecordTime, 1))
            return;
    }
}

const QMetaObject *QvkMainWindow_wl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QvkMainWindow_wl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QvkMainWindow_wlE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int QvkMainWindow_wl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void QvkMainWindow_wl::signal_newVideoFilename(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QvkMainWindow_wl::signal_beginRecordTime(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
