/********************************************************************************
** Form generated from reading UI file 'QvkDirDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QVKDIRDIALOG_H
#define UI_QVKDIRDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_QvkDirDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTreeView *treeView;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *QvkDirDialog)
    {
        if (QvkDirDialog->objectName().isEmpty())
            QvkDirDialog->setObjectName("QvkDirDialog");
        QvkDirDialog->resize(831, 541);
        verticalLayout = new QVBoxLayout(QvkDirDialog);
        verticalLayout->setObjectName("verticalLayout");
        treeView = new QTreeView(QvkDirDialog);
        treeView->setObjectName("treeView");

        verticalLayout->addWidget(treeView);

        buttonBox = new QDialogButtonBox(QvkDirDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(QvkDirDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, QvkDirDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, QvkDirDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(QvkDirDialog);
    } // setupUi

    void retranslateUi(QDialog *QvkDirDialog)
    {
        QvkDirDialog->setWindowTitle(QCoreApplication::translate("QvkDirDialog", "Dialog", nullptr));
    } // retranslateUi

};

namespace Ui {
    class QvkDirDialog: public Ui_QvkDirDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QVKDIRDIALOG_H
