/********************************************************************************
** Form generated from reading UI file 'licenses.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LICENSES_H
#define UI_LICENSES_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_license
{
public:
    QVBoxLayout *verticalLayout_2;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QSpacerItem *horizontalSpacer;
    QLabel *label_dummy_1;
    QLabel *label_dummy_3;
    QLabel *label_dummy_4;
    QLabel *label_dummy_2;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *pushButtonClose;

    void setupUi(QWidget *license)
    {
        if (license->objectName().isEmpty())
            license->setObjectName("license");
        license->resize(800, 600);
        verticalLayout_2 = new QVBoxLayout(license);
        verticalLayout_2->setObjectName("verticalLayout_2");
        scrollArea = new QScrollArea(license);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 780, 548));
        verticalLayout = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 4, 1, 1);

        label_dummy_1 = new QLabel(scrollAreaWidgetContents);
        label_dummy_1->setObjectName("label_dummy_1");
        label_dummy_1->setText(QString::fromUtf8("TextLabel"));

        gridLayout->addWidget(label_dummy_1, 0, 0, 1, 1);

        label_dummy_3 = new QLabel(scrollAreaWidgetContents);
        label_dummy_3->setObjectName("label_dummy_3");
        label_dummy_3->setText(QString::fromUtf8("TextLabel"));

        gridLayout->addWidget(label_dummy_3, 0, 2, 1, 1);

        label_dummy_4 = new QLabel(scrollAreaWidgetContents);
        label_dummy_4->setObjectName("label_dummy_4");
        label_dummy_4->setText(QString::fromUtf8("TextLabel"));

        gridLayout->addWidget(label_dummy_4, 0, 3, 1, 1);

        label_dummy_2 = new QLabel(scrollAreaWidgetContents);
        label_dummy_2->setObjectName("label_dummy_2");
        label_dummy_2->setText(QString::fromUtf8("TextLabel"));

        gridLayout->addWidget(label_dummy_2, 0, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_2->addWidget(scrollArea);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        pushButtonClose = new QPushButton(license);
        pushButtonClose->setObjectName("pushButtonClose");
        pushButtonClose->setText(QString::fromUtf8(""));

        horizontalLayout->addWidget(pushButtonClose);


        verticalLayout_2->addLayout(horizontalLayout);


        retranslateUi(license);

        QMetaObject::connectSlotsByName(license);
    } // setupUi

    void retranslateUi(QWidget *license)
    {
        license->setWindowTitle(QCoreApplication::translate("license", "Copyright and license from pictures", nullptr));
    } // retranslateUi

};

namespace Ui {
    class license: public Ui_license {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LICENSES_H
