/********************************************************************************
** Form generated from reading UI file 'QvkAudioPipewireSingle_wl.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QVKAUDIOPIPEWIRESINGLE_WL_H
#define UI_QVKAUDIOPIPEWIRESINGLE_WL_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_QvkAudioPipewireSingle_wl
{
public:
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QCheckBox *checkBoxAudioDevice;
    QVBoxLayout *verticalLayout;
    QLabel *labelAudioDevice;
    QProgressBar *progressBarAudioDevice;
    QToolButton *toolButtonAudioLevelmeter;

    void setupUi(QFrame *QvkAudioPipewireSingle_wl)
    {
        if (QvkAudioPipewireSingle_wl->objectName().isEmpty())
            QvkAudioPipewireSingle_wl->setObjectName("QvkAudioPipewireSingle_wl");
        QvkAudioPipewireSingle_wl->resize(400, 22);
        QvkAudioPipewireSingle_wl->setWindowTitle(QString::fromUtf8(""));
        QvkAudioPipewireSingle_wl->setLineWidth(0);
        verticalLayout_2 = new QVBoxLayout(QvkAudioPipewireSingle_wl);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(6, -1, 6, -1);
        checkBoxAudioDevice = new QCheckBox(QvkAudioPipewireSingle_wl);
        checkBoxAudioDevice->setObjectName("checkBoxAudioDevice");
        checkBoxAudioDevice->setText(QString::fromUtf8(""));

        horizontalLayout->addWidget(checkBoxAudioDevice);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(1);
        verticalLayout->setObjectName("verticalLayout");
        labelAudioDevice = new QLabel(QvkAudioPipewireSingle_wl);
        labelAudioDevice->setObjectName("labelAudioDevice");
        labelAudioDevice->setMaximumSize(QSize(16777215, 12));
        labelAudioDevice->setText(QString::fromUtf8("TextLabel"));

        verticalLayout->addWidget(labelAudioDevice);

        progressBarAudioDevice = new QProgressBar(QvkAudioPipewireSingle_wl);
        progressBarAudioDevice->setObjectName("progressBarAudioDevice");
        progressBarAudioDevice->setMaximumSize(QSize(16777215, 4));
        progressBarAudioDevice->setValue(24);
        progressBarAudioDevice->setTextVisible(false);

        verticalLayout->addWidget(progressBarAudioDevice);


        horizontalLayout->addLayout(verticalLayout);

        toolButtonAudioLevelmeter = new QToolButton(QvkAudioPipewireSingle_wl);
        toolButtonAudioLevelmeter->setObjectName("toolButtonAudioLevelmeter");
        toolButtonAudioLevelmeter->setMaximumSize(QSize(20, 20));
        toolButtonAudioLevelmeter->setText(QString::fromUtf8(""));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/pictures/screencast/missing.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        icon.addFile(QString::fromUtf8(":/pictures/screencast/accept.png"), QSize(), QIcon::Mode::Normal, QIcon::State::On);
        toolButtonAudioLevelmeter->setIcon(icon);
#if QT_CONFIG(shortcut)
        toolButtonAudioLevelmeter->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        toolButtonAudioLevelmeter->setCheckable(true);
        toolButtonAudioLevelmeter->setChecked(true);

        horizontalLayout->addWidget(toolButtonAudioLevelmeter);


        verticalLayout_2->addLayout(horizontalLayout);


        retranslateUi(QvkAudioPipewireSingle_wl);

        QMetaObject::connectSlotsByName(QvkAudioPipewireSingle_wl);
    } // setupUi

    void retranslateUi(QFrame *QvkAudioPipewireSingle_wl)
    {
#if QT_CONFIG(shortcut)
        checkBoxAudioDevice->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        (void)QvkAudioPipewireSingle_wl;
    } // retranslateUi

};

namespace Ui {
    class QvkAudioPipewireSingle_wl: public Ui_QvkAudioPipewireSingle_wl {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QVKAUDIOPIPEWIRESINGLE_WL_H
