/********************************************************************************
** Form generated from reading UI file 'help.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HELP_H
#define UI_HELP_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_help
{
public:
    QVBoxLayout *verticalLayout_2;
    QTextBrowser *textBrowser;
    QHBoxLayout *horizontalLayout;
    QLabel *labelURL;
    QPushButton *pushButtonClose;

    void setupUi(QWidget *help)
    {
        if (help->objectName().isEmpty())
            help->setObjectName("help");
        help->resize(400, 300);
        help->setWindowTitle(QString::fromUtf8("Form"));
        help->setWindowFilePath(QString::fromUtf8(""));
        verticalLayout_2 = new QVBoxLayout(help);
        verticalLayout_2->setObjectName("verticalLayout_2");
        textBrowser = new QTextBrowser(help);
        textBrowser->setObjectName("textBrowser");
        textBrowser->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(textBrowser);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        labelURL = new QLabel(help);
        labelURL->setObjectName("labelURL");
#if QT_CONFIG(tooltip)
        labelURL->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)
        labelURL->setText(QString::fromUtf8("TextLabel"));
        labelURL->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        horizontalLayout->addWidget(labelURL);

        pushButtonClose = new QPushButton(help);
        pushButtonClose->setObjectName("pushButtonClose");
        pushButtonClose->setText(QString::fromUtf8(""));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/pictures/screencast/exit.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushButtonClose->setIcon(icon);
        pushButtonClose->setIconSize(QSize(32, 22));

        horizontalLayout->addWidget(pushButtonClose);

        horizontalLayout->setStretch(0, 1);

        verticalLayout_2->addLayout(horizontalLayout);


        retranslateUi(help);

        QMetaObject::connectSlotsByName(help);
    } // setupUi

    void retranslateUi(QWidget *help)
    {
        (void)help;
    } // retranslateUi

};

namespace Ui {
    class help: public Ui_help {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HELP_H
