/********************************************************************************
** Form generated from reading UI file 'playerGST.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PLAYERGST_H
#define UI_PLAYERGST_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_playerGST
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label_logo;
    QWidget *widget_menuebar;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_3;
    QLabel *label_playbackTime;
    QLabel *label_3;
    QLabel *label_duration;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushButtonPlay;
    QPushButton *pushButtonPause;
    QPushButton *pushButtonStop;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushButtonFrameBackward;
    QPushButton *pushButtonFrameForward;
    QSpacerItem *horizontalSpacer_4;
    QPushButton *pushButtonOpenFile;
    QSpacerItem *horizontalSpacer_5;
    QPushButton *pushButtonMute;

    void setupUi(QWidget *playerGST)
    {
        if (playerGST->objectName().isEmpty())
            playerGST->setObjectName("playerGST");
        playerGST->resize(800, 600);
        playerGST->setWindowTitle(QString::fromUtf8(""));
        verticalLayout = new QVBoxLayout(playerGST);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        label_logo = new QLabel(playerGST);
        label_logo->setObjectName("label_logo");
        label_logo->setText(QString::fromUtf8(""));
        label_logo->setPixmap(QPixmap(QString::fromUtf8(":/pictures/logo/logo.png")));
        label_logo->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(label_logo);

        widget_menuebar = new QWidget(playerGST);
        widget_menuebar->setObjectName("widget_menuebar");
        verticalLayout_3 = new QVBoxLayout(widget_menuebar);
        verticalLayout_3->setSpacing(0);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(10, 0, 10, 0);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_3);

        label_playbackTime = new QLabel(widget_menuebar);
        label_playbackTime->setObjectName("label_playbackTime");
        label_playbackTime->setText(QString::fromUtf8("00:00:00"));

        horizontalLayout_2->addWidget(label_playbackTime);

        label_3 = new QLabel(widget_menuebar);
        label_3->setObjectName("label_3");
        label_3->setText(QString::fromUtf8(" / "));

        horizontalLayout_2->addWidget(label_3);

        label_duration = new QLabel(widget_menuebar);
        label_duration->setObjectName("label_duration");
        label_duration->setText(QString::fromUtf8("00:00:00"));

        horizontalLayout_2->addWidget(label_duration);


        verticalLayout_3->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(7);
        horizontalLayout->setObjectName("horizontalLayout");
        pushButtonPlay = new QPushButton(widget_menuebar);
        pushButtonPlay->setObjectName("pushButtonPlay");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/pictures/player/start.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushButtonPlay->setIcon(icon);

        horizontalLayout->addWidget(pushButtonPlay);

        pushButtonPause = new QPushButton(widget_menuebar);
        pushButtonPause->setObjectName("pushButtonPause");
        pushButtonPause->setEnabled(true);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/pictures/player/pause.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushButtonPause->setIcon(icon1);

        horizontalLayout->addWidget(pushButtonPause);

        pushButtonStop = new QPushButton(widget_menuebar);
        pushButtonStop->setObjectName("pushButtonStop");
        pushButtonStop->setEnabled(true);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/pictures/player/stop.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushButtonStop->setIcon(icon2);

        horizontalLayout->addWidget(pushButtonStop);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pushButtonFrameBackward = new QPushButton(widget_menuebar);
        pushButtonFrameBackward->setObjectName("pushButtonFrameBackward");
        pushButtonFrameBackward->setEnabled(false);
        pushButtonFrameBackward->setText(QString::fromUtf8(""));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/pictures/player/go-previous.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushButtonFrameBackward->setIcon(icon3);

        horizontalLayout->addWidget(pushButtonFrameBackward);

        pushButtonFrameForward = new QPushButton(widget_menuebar);
        pushButtonFrameForward->setObjectName("pushButtonFrameForward");
        pushButtonFrameForward->setEnabled(false);
        pushButtonFrameForward->setText(QString::fromUtf8(""));
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/pictures/player/go-next.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushButtonFrameForward->setIcon(icon4);

        horizontalLayout->addWidget(pushButtonFrameForward);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_4);

        pushButtonOpenFile = new QPushButton(widget_menuebar);
        pushButtonOpenFile->setObjectName("pushButtonOpenFile");
        pushButtonOpenFile->setText(QString::fromUtf8(""));
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/pictures/player/document-open.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushButtonOpenFile->setIcon(icon5);

        horizontalLayout->addWidget(pushButtonOpenFile);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_5);

        pushButtonMute = new QPushButton(widget_menuebar);
        pushButtonMute->setObjectName("pushButtonMute");
        pushButtonMute->setText(QString::fromUtf8(""));
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/pictures/player/audio-volume-high.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pushButtonMute->setIcon(icon6);
        pushButtonMute->setCheckable(true);

        horizontalLayout->addWidget(pushButtonMute);


        verticalLayout_3->addLayout(horizontalLayout);


        verticalLayout->addWidget(widget_menuebar);

        verticalLayout->setStretch(0, 1);

        retranslateUi(playerGST);

        QMetaObject::connectSlotsByName(playerGST);
    } // setupUi

    void retranslateUi(QWidget *playerGST)
    {
        pushButtonPlay->setText(QCoreApplication::translate("playerGST", "Play", nullptr));
        pushButtonPause->setText(QCoreApplication::translate("playerGST", "Pause", nullptr));
        pushButtonStop->setText(QCoreApplication::translate("playerGST", "Stop", nullptr));
        (void)playerGST;
    } // retranslateUi

};

namespace Ui {
    class playerGST: public Ui_playerGST {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PLAYERGST_H
